# Distributed Key‑Value Store – Atividade 2

> **Stack**: Python 3.12 · FastAPI · Redis (cache) · RabbitMQ (queue) · SQLite/CockroachDB (storage) · Nginx (LB) · Docker Compose

## 1. Subir o cluster localmente

```bash
git clone <repo>
cd distributed-kv
./start.sh               # ⚡ pull, build e `docker compose up -d`
```

A API ficará disponível em **http://localhost**.

## 2. End‑points

| Método | Rota          | Corpo / Query                         | Descrição                    |
|--------|---------------|---------------------------------------|------------------------------|
| GET    | `/kv/{key}` | —                                     | Obtém valor                  |
| PUT    | `/kv/{key}` | `{ "value": "<string>" }`           | Regista / actualiza valor    |
| DELETE | `/kv/{key}` | —                                     | Remove chave                 |
| GET    | `/`           | —                                     | Health‑check (“status”)      |

### Exemplos `curl`

```bash
curl -X PUT -H "Content-Type: application/json" \
     -d '{"value":"hello"}' http://localhost/kv/foo

curl http://localhost/kv/foo                 # {"data":{"value":"hello"}}
curl -X DELETE http://localhost/kv/foo
```

## 3. Arquitectura

Veja o diagrama no enunciado (*slide “Exemplo key‑value API” – Aula 12*).

Fluxo resumido:

1. **Cliente → LB (Nginx)** – balanceamento round‑robin.
2. **API (FastAPI)** publica em *RabbitMQ* (“add_key” / “del_key”).
3. **Consumer** aplica lógica de *upsert* / delete na BD (**SQLite / Cockroach**) e sincroniza a **cache Redis**.
4. Leituras **GET** consultam primeiro o Redis, caindo na BD apenas em *cache miss*.

## 4. Consistência

* Timestamp (`time_ns`) acompanha cada operação.  
* `consumer.py` só substitui / remove valores se o `last_updated` armazenado for **≤ timestamp** recebido.  
* CockroachDB providencia replicação **Raft** quando escalado para 3 nós.

## 5. Testes de carga

Exemplo rápido com **ab**:

```bash
ab -n 1000 -c 50 http://localhost/kv/ping
```

Troque por **JMeter** / **k6** para relatório detalhado.

---

📄 *Documento gerado automaticamente em 2025-05-20*.
