import os, json, asyncio, aioredis

REDIS_HOST = os.getenv("REDIS_HOST", "redis")
TTL_SECONDS = int(os.getenv("CACHE_TTL", "300"))

_redis = None

async def init():
    global _redis
    _redis = await aioredis.from_url(f"redis://{REDIS_HOST}", decode_responses=True)

async def get(key: str):
    if _redis is None:
        return None
    return await _redis.get(key)

async def set(key: str, value: str):
    if _redis is None:
        return
    await _redis.set(key, value, ex=TTL_SECONDS)

async def delete(key: str):
    if _redis is None:
        return
    await _redis.delete(key)
