import os, time
from sqlalchemy import create_engine, select, insert, update, delete
from sqlalchemy.orm import Session
from .models import Base, KV

DB_DSN = os.getenv("DB_DSN", "sqlite:///data/app.db")
_engine = create_engine(DB_DSN, connect_args={"check_same_thread": False})

def init_sync():
    Base.metadata.create_all(_engine)

def _now_ns() -> int:
    return int(time.time_ns())

def get(key: str):
    with Session(_engine) as s:
        stmt = select(KV).where(KV.key == key)
        return s.scalars(stmt).first()

def upsert_if_newer(key: str, value: str, ts: int) -> bool:
    """Return True if write executed"""
    with Session(_engine) as s:
        rec = s.get(KV, key)
        if rec is None:
            s.add(KV(key=key, value=value, last_updated=ts))
            s.commit()
            return True
        if rec.last_updated <= ts:
            rec.value = value
            rec.last_updated = ts
            s.commit()
            return True
        return False

def delete_if_newer(key: str, ts: int) -> bool:
    with Session(_engine) as s:
        rec = s.get(KV, key)
        if rec and rec.last_updated <= ts:
            s.delete(rec)
            s.commit()
            return True
        return False
