import os, json, pika, backoff

RABBIT_URL = os.getenv("RABBIT_URL", "amqp://guest:guest@rabbit1:5672/")
QUEUE_ADD = "add_key"
QUEUE_DEL = "del_key"

_connection = None
_channel = None

def _connect():
    global _connection, _channel
    _connection = pika.BlockingConnection(pika.URLParameters(RABBIT_URL))
    _channel = _connection.channel()
    _channel.queue_declare(QUEUE_ADD, durable=True)
    _channel.queue_declare(QUEUE_DEL, durable=True)

@backoff.on_exception(backoff.expo, Exception, max_tries=5)
def init():
    _connect()

def publish(queue: str, payload: dict):
    if _channel is None:
        init()
    _channel.basic_publish(
        exchange="",
        routing_key=queue,
        body=json.dumps(payload).encode(),
        properties=pika.BasicProperties(delivery_mode=2)  # make message persistent
    )
